#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

#include "interface.h"
#include "support.h"

static GtkFileSelection *fsel; /* need to pass Params from button callback */
static char fsellock;

void okAction(GtkWidget *button, gpointer user_data)
{
   printf ("%s\n", gtk_file_selection_get_filename (fsel));
}

void clearLockAction(GtkWidget *button, gpointer user_data)
{
   fsellock = FALSE; /* release selector for a new call */
}

void storeNameAction(GtkWidget *button, gpointer user_data)
{
   gchar **result = (gchar **)user_data; /* need pointer to pointer ! */
   
   ldebug(4,"storeNameAction: result=%ld\n", result);
   *result = (gchar *)gtk_file_selection_get_filename (fsel);

   clearLockAction(button, user_data);
}

void newFileSelector(char *preset, char **result, gpointer ok_action,
			gpointer cancel_action, gpointer user_data)
{
   ldebug(2,"newFileSelector entered\n");
   ldebug(4,"newFileSelector: result=%ld\n", result);
   
   if(fsellock == TRUE)
   {
      writeLog("Fileselector locked by an unfinished previous call !\n");
      return;
   }
   fsellock = TRUE;
   
   *result = NULL; /* clear possible leftover from previous call */
  
   fsel = GTK_FILE_SELECTION(gtk_file_selection_new ("File selector"));
   
   /* Connect ok_button to first store the filename and then call ok_action */
   gtk_signal_connect (GTK_OBJECT (fsel->ok_button), "clicked",
		       (GtkSignalFunc) storeNameAction, result);
   gtk_signal_connect (GTK_OBJECT (fsel->ok_button), "clicked",
		       (GtkSignalFunc) ok_action, user_data );
   
   /* Connect cancel_button to first release lock and then call cancel_action
    if non NULL */
   gtk_signal_connect (GTK_OBJECT (fsel->cancel_button), "clicked",
		       (GtkSignalFunc) clearLockAction, fsel );
   if(cancel_action != NULL)
     gtk_signal_connect (GTK_OBJECT (fsel->cancel_button), "clicked",
			 (GtkSignalFunc) cancel_action, fsel );
   
   /* Destroy fileselector after either ok or cancel is clicked */
   gtk_signal_connect_object (GTK_OBJECT (fsel->ok_button),
			      "clicked", (GtkSignalFunc) gtk_widget_destroy,
			      GTK_OBJECT (fsel));
   gtk_signal_connect_object (GTK_OBJECT (fsel->cancel_button),
			      "clicked", (GtkSignalFunc) gtk_widget_destroy,
			      GTK_OBJECT (fsel));
   
   /* Set preselected filename if supplied */
   if(preset != NULL)
     gtk_file_selection_set_filename (fsel, preset);

   gtk_widget_show(GTK_WIDGET(fsel));
}

