#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

int getSortKey(void)
{
   int y;
   Params *p = getParams();
   
   ldebug(2,"getSortKey called\n");
   
   /* Find unique primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags)&&IS_UNIQUE(p->fields[y].flags))
      {
	 writeLog("Using unique primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find primary key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_PRI_KEY(p->fields[y].flags))
      {
	 writeLog("Using primary key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   /* Else, find first unique key */
   for(y=0;y<p->numflds;y++)
   {
      if(IS_UNIQUE(p->fields[y].flags))
      {
	 writeLog("Using unique key %d for sort in %s\n",y, p->table);
	 return y;
      }
   }
   
   writeLog("No unique and/or primary key in %s,"
	    "using first field (0).\n", p->table);
   return 0;
}


void setSortKey(int row)
{
   Params *p = getParams();
   p->sortkey = row;
   refreshFieldTab();
   p->tabinfo[RECORDSTAB].refresh = TRUE; // force a redraw for records if record tab is selected
}

/* If we arrive here it means the user has doubleclicked a row in the list. */
void field_doubleclick(GtkTreeView       *tree_view,
		       GtkTreePath       *path,
		       GtkTreeViewColumn *column,
		       gpointer           data)
{
   Params *p = getParams();
   TabInfo *ti = & p->tabinfo[FIELDSTAB];
   gint* indices;
   
   ldebug(2,"field_doubleclick entered\n");

   indices = gtk_tree_path_get_indices (path);
   ti->selectedrow=indices[0];
   
   ldebug(3,"field_doubleclick: you doubleclicked row %d.\n", ti->selectedrow);
   
   if(ti->table == NULL)
   {
      showMessage("Currently no table selected !\n");
      return;
   }
   writeLog("Setting sort field to %d (%s)\n",
	    ti->selectedrow,ti->fields[ti->selectedrow].name);
   /* watch out: next routine also forces a refresh of the tab !! */
   setSortKey(ti->selectedrow);
}

/* This is called for each cell when rendering it on the screen.
 * It is used to set the sortkey color to a different value than the others.
*/
void color_fields_sortkey_func (GtkTreeViewColumn *col,
				GtkCellRenderer   *renderer,
				GtkTreeModel      *model,
				GtkTreeIter       *iter,
				gpointer           user_data)
{
   GtkTreePath *sortkeypath = (GtkTreePath *)user_data;
   GtkTreePath *currentpath = gtk_tree_model_get_path (model, iter);
   
   ldebug(6,"color_fields_sortkey_func entered\n");
   ldebug(6,"sortkeypath %s, currentpath %s\n",
	  gtk_tree_path_to_string (sortkeypath),
	  gtk_tree_path_to_string (currentpath));
   
   /* make red if we are rendering the sortkey */
   if(gtk_tree_path_compare(sortkeypath, currentpath) == 0)
     g_object_set(renderer,
		  "foreground", "Red",
		  "foreground-set", TRUE,
		  NULL);
   else
     g_object_set(renderer,
		  "foreground", "Blue",
		  "foreground-set", TRUE,
		  NULL);
}

gchar *getFieldTypeString(Params *p, int index)
{
   ldebug(2,"getFieldTypeString entered : fieldtype %d\n",p->fields[index].type);
   
   /* Next uses fields deprecated since MySQL 4.0 but still necessary for correct results */
   
   if(p->fields[index].flags & ENUM_FLAG ) return "Enum";
   if(p->fields[index].flags & BLOB_FLAG ) return "Blob";
   if(p->fields[index].flags & SET_FLAG  ) return "Set";
   if(p->fields[index].flags & TIMESTAMP_FLAG ) return "Timestamp";
   
   switch(p->fields[index].type)
   {
    case FIELD_TYPE_STRING:     return "String";    break;
    case FIELD_TYPE_VAR_STRING: return "VarString"; break;
    case FIELD_TYPE_FLOAT:      return "Float";     break;
    case FIELD_TYPE_DOUBLE:     return "Double";    break;
    case FIELD_TYPE_DECIMAL:    return "Decimal";   break;
    case FIELD_TYPE_LONGLONG:
    case FIELD_TYPE_INT24:
    case FIELD_TYPE_TINY:
    case FIELD_TYPE_SHORT:
    case FIELD_TYPE_LONG:       return "Integer";   break;
    case FIELD_TYPE_DATE:       return "Date";      break;
    case FIELD_TYPE_TIME:       return "Time";      break;
    case FIELD_TYPE_TIMESTAMP:  return "TimeStamp"; break;
    case FIELD_TYPE_DATETIME:   return "Datetime";  break;
    case FIELD_TYPE_YEAR:       return "Year";      break;
    case FIELD_TYPE_SET:        return "Set";       break;
    case FIELD_TYPE_ENUM:       return "Enum";      break;
    case FIELD_TYPE_TINY_BLOB:
    case FIELD_TYPE_MEDIUM_BLOB:
    case FIELD_TYPE_LONG_BLOB:
    case FIELD_TYPE_BLOB:       return "Blob:";     break;
    default:
      writeLog("Unknown fieldtype %d detected !\n",
	       p->fields[index].type);
      return "Unknown"; break;
   }
}

#define FIELDCOLS 8 /* Number of columns (this is here not the same as nrflds !!) */
void showFields(void)
{
   int   x,y;
   TabInfo *ti;
   GtkTreeIter iter;
   Params *p = getParams();
   gchar lengthstring[10];
   gchar decimalstring[10];
   gchar *data[FIELDCOLS];
   gchar *titles[FIELDCOLS] =
   { "Fieldname", "Tablename", "Length", "Type", "Default", "Decimals", "Unique", "Not Null" };
   
   ldebug(2,"showFields entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show fields: select a table first !\n");
      return;
   }
   
   ti = initTabInfo(FIELDSTAB, FIELDCOLS, p->numflds);
   
   /* Now set the fieldnames for the column titles (before calling initTabList) */
   for(x=0;x<FIELDCOLS;x++)
     ti->titles[x] = titles[x];
   
   initTabList(ti, FIELDSTAB);
   
   for(y=0; y < ti->nrrows; y++)
   {
      GValue field = {0}; // bogus value
      
      /* convert field length and nr. of decimals to string: */
      snprintf(lengthstring,sizeof(lengthstring) - 1,"%8lu",p->fields[y].length);
      snprintf(decimalstring,sizeof(decimalstring) - 1,"%8d",p->fields[y].decimals);
      
      data[0] = p->fields[y].name;
      data[1] = p->table;
      data[2] = lengthstring;
      data[3] = getFieldTypeString(p, y);
      data[4] = p->fields[y].def;
      data[5] = decimalstring;
      data[6] = IS_UNIQUE(p->fields[y].flags) ? "Yes" : "No";
      data[7] = IS_NOT_NULL(ti->fields[y].flags) ? "Yes" : "No";
      
      /* append a new row to the list in the tab : */
      gtk_list_store_append(ti->list, &iter);
      ldebug(5,"showFields: storing row %d \n",y);
      
      g_value_init (&field, G_TYPE_STRING);
      
      for(x=0;x < ti->nrcols;x++)
      {
         ldebug(6,"showFields: storing field %d,%d (%s) \n",y,x,data[x]);
	 g_value_set_string (&field,data[x]);
	 gtk_list_store_set_value (ti->list, &iter, x, &field);
      }
      
      g_value_unset (&field);
   }
   /* This is to change the color for the sortkey row (!!). As we
    * have a renderer for each column but NOT for each cell we need 
    * a dynamic function to set each element to the appropriate color.
    */
   for(x=0;x < ti->nrcols;x++)
   {
      gtk_tree_view_column_set_cell_data_func
	(gtk_tree_view_get_column(ti->view, x), ti->renderer[x],
	 color_fields_sortkey_func,
	 gtk_tree_path_new_from_indices (ti->sortkey,-1),NULL);
   }
   
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), FIELDSTAB);
   ti->refresh = FALSE;
}

