#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* If we come here, then the user has selected a row in the list. */
void qrecord_doubleclick(GtkTreeView       *tree_view,
			 GtkTreePath       *path,
			 GtkTreeViewColumn *column,
			 gpointer           data)
{
   Params *p = getParams();
   TabInfo *ti = & p->tabinfo[QUERYTAB];
   gint* indices;
   
   ldebug(2,"qrecord_doubleclick entered\n");
   
   indices = gtk_tree_path_get_indices (path);
   ti->selectedrow=indices[0];
   
   ldebug(2,"qrecord_doubleclick: you doubleclicked row %d.\n",ti->selectedrow);
   
   showMessage("Sorry: can't open free query records\n"
	       "in a separate window !");
}

/* TODO : combine with showRecords ? */
void showQuery(char *query)
{
   int     x,y;
   SQL_RES *res;
   TabInfo *ti;
   GtkTreeIter iter;
   Params *p = getParams();
   
   ldebug(2,"showQuery entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show select query:\n select a table first !");
      return;
   }
   
   if(strncasecmp(query,"select ",7) != 0)
   {
      showMessage("Cannot execute query: Currently only\n"
		  "SELECT statements allowed. Sorry !");
      return;
   }
   
   if (executeQuery(p->dbsock, query))
   {
      showSqlError(p->dbsock,"Query in showQuery failed !");
      return;
   }
   
   res = sql_store_result(p->dbsock);
   ti  = initTabInfo(QUERYTAB, sql_num_fields(res), sql_num_rows(res));
   
   /* Now set the fieldnames for the column titles (before calling initTabList) */
   for(x=0;x<ti->nrcols;x++)
     ti->titles[x] = res->fields[x].name;
   
   initTabList(ti, QUERYTAB);
   
   // and populate the list inside the treeview :
   for(y=0;y < ti->nrrows;y++)
   {
      
      SQL_ROW row;
      GValue field = {0} ; // bogus value
      
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(ti->dbsock,"Error reading record %d !",y);
	 return;
      }
      
      /* append a new row to the list in the tab : */
      gtk_list_store_append(ti->list, &iter);
      ldebug(5,"showQuery: storing row %d \n",y);
      
      g_value_init (&field, G_TYPE_STRING);
      
      for(x=0;x < ti->nrcols;x++)
      {
	 ldebug(6,"showQuery: storing field %d,%d (%s) \n",y,x,row[x]);
	 g_value_set_string (&field,row[x]);
	 gtk_list_store_set_value (ti->list, &iter, x, &field);
      }
      
      g_value_unset (&field);
   }   
   /* We don't have a (known) sortkey here, so we simply change the 
    * color for all column renderers to blue :
    */
   for(x=0;x < ti->nrcols;x++)
   {
      g_object_set(ti->renderer[x],
		   "foreground", "Blue",
		   "foreground-set", TRUE,
		   NULL);
   }
   
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), QUERYTAB);
   ti->refresh = FALSE;
}

void loadQueryAction(GtkWidget *w, gpointer user_data)
{
   FILE *input;
   char qline[MAXQUERY];
   Params *p = getParams();
   GtkTextIter    txtstart,txtend;
   
   ldebug(2,"loadQueryAction entered\n");
   
   input = fopen(p->selectedname,"r");
   if(input == NULL)
   {
      showVarArgMessage("Cannot load query:\n open file %s failed !",
			p->selectedname);
      return;
   }
   /* clear current buffer : */
   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   gtk_text_buffer_get_end_iter (p->querybuffer, &txtend);
   gtk_text_buffer_delete(p->querybuffer, &txtstart, &txtend);

   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   while(fgets(qline,MAXQUERY-1,input) != NULL)
     gtk_text_buffer_insert (p->querybuffer, &txtstart, qline, -1);

   fclose(input);
}

void saveQueryAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   Params *p = getParams();
   GtkTextIter    txtstart;
   GtkTextIter    txtend;
   
   ldebug(2,"saveQueryAction entered\n");
   
   output = fopen(p->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save query:\n open file %s failed !",
		  p->selectedname);
      return;
   }
   gtk_text_buffer_get_start_iter (p->querybuffer, &txtstart);
   gtk_text_buffer_get_end_iter   (p->querybuffer, &txtend);
   fputs(gtk_text_buffer_get_text(p->querybuffer,&txtstart,&txtend,FALSE),output);
   
   fclose(output);
}

