#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* FIX ME: use record_selected instead or do we need more checks ? */
/* If we arrive here, then the user has selected a row in the list. */
void srecord_doubleclick(GtkTreeView       *tree_view,
			 GtkTreePath       *path,
			 GtkTreeViewColumn *column,
			 gpointer           data)
{
   Params *p = getParams();
   TabInfo *ti = & p->tabinfo[SEARCHTAB];
   gint* indices;
   
   ldebug(2,"srecord_doubleclick entered\n");
   
   indices = gtk_tree_path_get_indices (path);
   ti->selectedrow=indices[0];

   ldebug(3,"srecord_doubleclick: you doubleclicked row %d.\n", ti->selectedrow);

   if(ti->table == NULL)
   {
      showMessage("Currently no table selected !");
      return;
   }
   
   newModifyWindow(ti);
}

char *buildSearchQuery(Record *rec)
{
   char *query;
   int x,y;
   int size = 100; /* safe start */
   
   ldebug(2,"buildSearchQuery entered\n");
   
   /* get approximate querylength: account for quotes,spaces and equalsigns */
   for(y=0;y<rec->numflds;y++)
     size += strlen(rec->fields[y].name) + rec->fields[y].length + 10;
   query =(char *)g_malloc(size * sizeof(char));
   
   snprintf(query, size - 1, SELECT WHERE, rec->table);
   
   x=0;
   for(y=0;y<rec->numflds;y++)
   {
      gchar *entry = (gchar *)gtk_entry_get_text(rec->entries[y]);
      
      if(strlen(entry) > 0)
      {	 
	 if(x++ > 0) strcat(query, AND);
	 strcat (query, rec->fields[y].name);
	 strcat (query, LIKE);
	 strcat (query, addStringQuotes(rec->fields[y].type));
	 strncat(query, entry, rec->fields[y].length);
	 strcat (query, addStringQuotes(rec->fields[y].type));
      }
   }
   strcat(query,ORDERBY);
   strcat(query,rec->fields[rec->sortkey].name);
   
   return query;
   /* don't forget to free query after use !! */
}

SQL_RES *selectSearch(Record *rec)
{
   int stat;
   char    *query;
   Params *p = getParams();
   
   query = buildSearchQuery(rec);
   
   stat = executeQuery(p->dbsock, query);
   free(query); /* buildSearchQuery mallocs, but relies on external free */
   
   if(stat != 0)
     return NULL;
   else
     return sql_store_result(p->dbsock);
}

/* TODO : combine with showRecords ? */
void showSearch(Record *rec)
{
   int     x,y;
   SQL_RES *res;
   TabInfo *ti;
   GtkTreeIter iter;
   Params *p = getParams();
   
   ldebug(2,"showSearch entered\n");
   if(p->table == NULL)
   {
      showMessage("Cannot show search matches:\n select a table first !");
      return;
   }
   
   if( ( res = selectSearch(rec) ) == NULL)
   {
      showSqlError(p->dbsock,"Select query for showSearch failed !");
      return;
   }
   
   ti = initTabInfo(SEARCHTAB, sql_num_fields(res), sql_num_rows(res));
   
   /* Now fetch the fieldnames for this selection : */
   for(x=0;x<ti->numflds;x++)
     ti->titles[x] = ti->fields[x].name;

   initTabList(ti, SEARCHTAB);
   
   // and populate the list inside the treeview :
   for(y=0;y < ti->nrrows;y++)
   {
      SQL_ROW row;
      GValue field = {0}; // bogus value
      
      if((row = sql_fetch_row(res)) == NULL)
      {
	 showSqlError(ti->dbsock,"Error reading record %d !",y);
	 return;
      }
      
      /* append a new row to the list in the tab : */
      gtk_list_store_append(ti->list, &iter);
      ldebug(5,"showSearch: storing row %d \n",y);
      
      g_value_init (&field, G_TYPE_STRING);
      
      for(x=0;x < ti->nrcols;x++)
      {
	 ldebug(6,"showSearch: storing field %d,%d (%s) \n",y,x,row[x]);
	 g_value_set_string (&field,row[x]);
	 gtk_list_store_set_value (ti->list, &iter, x, &field);
      }
      g_value_unset (&field);
      
   }
   /* This is only to change the color for the sortkey column
    * renderer to red and the others to blue :
    */
   for(x=0;x < ti->nrcols;x++)
   {
      if(x == ti->sortkey)
	g_object_set(ti->renderer[x],
		     "foreground", "Red",
		     "foreground-set", TRUE,
		     NULL);
      else
	g_object_set(ti->renderer[x],
		     "foreground", "Blue",
		     "foreground-set", TRUE,
		     NULL);
   }
   
   gtk_notebook_set_page (GTK_NOTEBOOK(p->notebook), SEARCHTAB);
   ti->refresh = FALSE;
}
