#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

/* If we arrive here it means a new row in the list was selected, either 
 * by a user click or by changing a tab, which will automatically select
 * row 0 */
void view_selected(GtkTreeSelection *selection, gpointer tabid)
{
   Params *p = getParams();
   TabInfo *ti = & p->tabinfo[(int)tabid];
   GtkTreeIter iter;
   GtkTreeModel *model;
   GtkTreePath* path;
   gint *indices;
   
   if (gtk_tree_selection_get_selected(selection,&model,&iter))
   {
      /* get the new row number by converting the selection path : */
      path = gtk_tree_model_get_path (model,&iter);
      indices = gtk_tree_path_get_indices (path);
      ti->selectedrow = indices[0];
      gtk_tree_path_free(path);
      
      ldebug (2,"You selected row %d in tab %d \n",ti->selectedrow,(int)tabid);
   }
}

void refreshRecordTab(void)
{
   GtkCheckMenuItem *auto_refresh =
     (GTK_CHECK_MENU_ITEM(lookup_widget (getTopWindow(),
					 "settings_auto_refresh")));

   ldebug(2,"refreshRecordTab entered\n");
   
   if(auto_refresh != NULL && auto_refresh->active != 0)
     forceRefreshRecordTab();
}

void forceRefreshRecordTab(void)
{
   Params *p = getParams();
   
   ldebug(2,"forceRefreshRecordTab entered\n");
   
   p->tabinfo[RECORDSTAB].refresh = TRUE; 
   // and call notebook callback
   on_notebooktab_switched
     (GTK_NOTEBOOK(lookup_widget(getTopWindow(), "notebook")),
      NULL, RECORDSTAB, NULL);
}

void refreshFieldTab(void)
{
   Params *p = getParams();
   ldebug(2,"refreshFieldTab entered\n");
   
   p->tabinfo[FIELDSTAB].refresh = TRUE; 
   // and call notebook callback
   on_notebooktab_switched
     (GTK_NOTEBOOK(lookup_widget(getTopWindow(), "notebook")),
      NULL, FIELDSTAB, NULL);
}

/* This should work for all available tabs */
void setTabColumnProperties(TabInfo *ti, int tabid)
{
   int x;
   int cwidth;
   GtkTreeViewColumn *column;
   
   ldebug(2,"setTabColumnProperties entered\n");
   
   /* set column sizes according to the fieldlength : */
   for(x=0;x<ti->nrcols;x++)
   {
      cwidth = (strlen(ti->titles[x]));
      switch(tabid)
      {
       case RECORDSTAB: 
       case SEARCHTAB :
       case QUERYTAB  :
	 if ( ti->fields[x].length > cwidth )
	   cwidth = ti->fields[x].length;
	 break;
       case FIELDSTAB :
	 cwidth += 8; /* slightly larger */
	 break;
      }
      
      cwidth *= FONTWIDTH;
      ldebug(5,"Column width for column %d : %d\n",x,cwidth);
      column = gtk_tree_view_get_column (ti->view, x);
      gtk_tree_view_column_set_min_width (column,cwidth);
      gtk_tree_view_column_set_resizable (column, TRUE);
   }
}

/* This should work for all available tabs (clists) */
void saveTabAction(GtkWidget *w, gpointer user_data)
{
   FILE *output;
   TabInfo *ti = (TabInfo *) user_data;
   GtkTreeIter iter;
   char *content;
   int x,y;
   
   ldebug(2,"saveTabAction entered\n");
   
   if(ti == NULL || ti->table == NULL)
   {
      showMessage("Cannot save records:\n no records in tab !");
      return;
   }
   
   output = fopen(ti->selectedname,"w");
   if(output == NULL)
   {
      showVarArgMessage("Cannot save records:\n open file %s failed !",
			ti->selectedname);
      return;
   }
   
   gtk_tree_model_get_iter_first (GTK_TREE_MODEL (ti->list), &iter);
   for(y=0;y<ti->nrrows; y++)
   {
      ldebug(3,"saveTabAction :");
      for(x=0;x<ti->nrcols;x++)
      {  
	 gtk_tree_model_get (GTK_TREE_MODEL (ti->list),&iter, x, &content, -1);
	 fprintf(output,"%s%s",content,DELIMITER);
	 ldebug(3,"%s%s",content,DELIMITER);
      }
      fprintf(output,"\n");
      gtk_tree_model_iter_next (GTK_TREE_MODEL (ti->list),&iter);
      ldebug(3,"\n");
   }
   
   fclose(output);
}

void clearTabList(TabInfo *ti)
{
   // make the list model empty :
   if(ti->list != NULL)
     gtk_list_store_clear (GTK_LIST_STORE(ti->list));
   
   // remove all columns in the treeview :
   while(1)
   {
      GtkTreeViewColumn *col;
      
      col = gtk_tree_view_get_column (ti->view, 0);
      if(col == NULL)
	break;
      gtk_tree_view_remove_column(ti->view, col );
   }
   
   // disassociate the list model with the treeview :
   gtk_tree_view_set_model (ti->view, NULL); 
   if(ti->list != NULL)
     g_object_unref (G_OBJECT (ti->list));
   ti->list = NULL;
}

void clearTabInfo(TabInfo *ti)
{
   ldebug(2,"clearTabInfo entered\n");
   
   if(ti->inuse == FALSE)
     return;
   
   g_free(ti->table);
   g_free(ti->renderer);

   freeFields(ti->fields,ti->numflds);
   ti->fields = NULL;
   
   ti->numflds = 0;
   ti->sortkey = -1;
   ti->dbsock = NULL;
   
   ti->nrcols     = 0;
   ti->nrrows     = 0;
   
   g_free(ti->titles);

   ti->selectedrow  = -1;
   ti->selectedname = NULL;

   clearTabList(ti);
   ti->inuse = FALSE;
   ti->refresh = FALSE;
}

void clearAllTabs(void)
{
   int x;
   Params *p = getParams();
   
   for(x=0;x<p->nrtabs;x++)
     clearTabInfo(&p->tabinfo[x]);
}

void initTabList(TabInfo *ti, int tabid)
{
   GType *types;
   int x;
   
   types = (GType *)g_malloc(ti->nrcols * sizeof(GType));

   ti->renderer = g_malloc(ti->nrcols *sizeof(GtkCellRenderer *));
   
   for(x=0;x < ti->nrcols; x++)
   {
      ldebug(3,"initTabList: adding column %d with title %s ...\n",x,ti->titles[x]);
      
      // create the renderer that sits behind the treeview column :
      ti->renderer[x] = gtk_cell_renderer_text_new ();
      // FIX ME : very simple row height adjustment, should be improved later.
      gtk_cell_renderer_set_fixed_size (ti->renderer[x],-1,17);
      
      gtk_tree_view_insert_column_with_attributes 
	(ti->view, -1, ti->titles[x], ti->renderer[x], "text", x, NULL);
      types[x] = G_TYPE_STRING; // currently all of the same type
   }
   
   // create the list that sits behind the treeview :
   ti->list = gtk_list_store_newv (ti->nrcols, types);
   
   gtk_tree_view_set_model(ti->view, GTK_TREE_MODEL(ti->list));
   
   g_free(types);
   setTabColumnProperties(ti, tabid);
}

/* Copy relevant information for each tab (view) in the notebook to a separate
 structure so each has its own private info. Note that there is no need to create
 the structs themselves as all of them are already defined in the Global Params (we
 know the number of tabs in the notebook at startuptime !) */
TabInfo *initTabInfo(int tabid, int nrcols, int nrrows)
{
   int tlen;
   TabInfo *ti;
   Params *p = getParams();
   
   ldebug(2,"initTabInfo entered\n");
   
   ti = &p->tabinfo[tabid];
   
   if(p->table == NULL)
   {
      fprintf(stderr,"initTabInfo: table in global Params struct is invalid !!\n");
      exit(1);
   }
   
   clearTabInfo(ti); // remove any previous leftovers 
   
   tlen = strlen(p->table) + 1;
   ti->table = (char *)g_malloc(tlen);
   strncpy(ti->table,p->table,tlen);
   
   ti->numflds    = p->numflds;
   ti->sortkey    = p->sortkey;
   ti->dbsock     = p->dbsock;

   /* Copy complete field array from TabInfo : */
   ti->fields = copyFields(p->fields, p->numflds);

   ti->nrcols     = nrcols; 
   ti->nrrows     = nrrows;

   /* Note that nrcols is not always the same as nrflds (eg in showFields) */
   ti->titles = (gchar **)g_malloc(sizeof(gchar *) * ti->nrcols);

   ti->selectedrow  = -1;
   ti->selectedname = NULL;
   
   ti->inuse   = TRUE;
   
   return ti;
}

